{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "# Membership functions"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 1,
   "metadata": {
    "collapsed": false
   },
   "outputs": [],
   "source": [
    "import numpy as np\n",
    "import matplotlib.pyplot as pl\n",
    "\n",
    "%matplotlib inline"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": [
    "class MembershipFunction:\n",
    "    \"\"\"\n",
    "    This is a base class providing functionnalities which are common to all membership functions.\n",
    "    \"\"\"\n",
    "    def plot(self, v_min, v_max, res=0.1):\n",
    "        input_values = np.arange(v_min, v_max, res)\n",
    "        output_values = map(self, input_values)\n",
    "        pl.plot(input_values, output_values)\n",
    "        pl.ylim((0, 1.1 * max(output_values)))\n",
    "        pl.grid()"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Some examples of membership functions"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "- Common membership functions are \"crisp\" (or Boolean), \"Triangular\", and \"Trapezoidal\"."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "metadata": {
    "collapsed": false
   },
   "outputs": [],
   "source": [
    "class CrispMF(MembershipFunction):\n",
    "    \"\"\"\n",
    "    Crisp rectangular membership function. Provided here to see the difference between crisp and fuzzy sets.\n",
    "    first_max: position of the rectangle\n",
    "    width: width of the rectangle (membership = 1)\n",
    "    \"\"\"\n",
    "    def __init__(self, first_max, width):\n",
    "        self.first_max = first_max\n",
    "        self.width = width\n",
    "    def __call__(self, value):\n",
    "        if value < self.first_max:\n",
    "            return 0.0\n",
    "        elif value > (self.first_max + self.width):\n",
    "            return 0.0\n",
    "        else:\n",
    "            return 1.0"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "metadata": {
    "collapsed": false,
    "scrolled": true
   },
   "outputs": [
    {
     "data": {
      "image/png": "iVBORw0KGgoAAAANSUhEUgAAAXcAAAD7CAYAAACRxdTpAAAABHNCSVQICAgIfAhkiAAAAAlwSFlz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\nepCau4hID/o/cVHUJgSoKcIAAAAASUVORK5CYII=\n",
      "text/plain": [
       "<matplotlib.figure.Figure at 0x7f80ec9baf10>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "c = CrispMF(3, 4)\n",
    "c.plot(0, 10)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 5,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": [
    "class TriangularMF(MembershipFunction):\n",
    "    \"\"\"\n",
    "    Triangular membership function. This function implements an isosceles triangle.\n",
    "    center: position of the center of the triangle\n",
    "    width: width of the base of the rectangle\n",
    "    \"\"\"\n",
    "    def __init__(self, center, width):\n",
    "        self.center = center\n",
    "        self.width = width\n",
    "        self.half_width = width / 2.0\n",
    "        self.slope = 1.0 / self.half_width\n",
    "        \n",
    "    def __call__(self, value):\n",
    "        if value < (self.center - self.half_width):\n",
    "            return 0.0\n",
    "        elif value > (self.center + self.half_width):\n",
    "            return 0.0\n",
    "        else:\n",
    "            return 1.0 - (np.abs(value - self.center) * self.slope)\n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "image/png": "iVBORw0KGgoAAAANSUhEUgAAAXcAAAD7CAYAAACRxdTpAAAABHNCSVQICAgIfAhkiAAAAAlwSFlz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\nYII=\n",
      "text/plain": [
       "<matplotlib.figure.Figure at 0x7f80ea892710>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "t = TriangularMF(3, 5)\n",
    "t.plot(0, 10)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": [
    "class TrapezoidalMF(MembershipFunction):\n",
    "    \"\"\"\n",
    "    Trapezoidal membership function. This function implements an isosceles trapezoid.\n",
    "    first_max: position of the first point where the membership function reach one (1)\n",
    "    width_base: width of the base of the trapezoid\n",
    "    width_top: widht of the top of the trapezoid\n",
    "    \"\"\"\n",
    "    def __init__(self, first_max, width_base, width_top):\n",
    "        self.first_max = first_max\n",
    "        self.width_base = width_base\n",
    "        self.width_top = width_top\n",
    "        self.base_triangle = (width_base - width_top) / 2.0\n",
    "        self.slope = 1.0 / self.base_triangle\n",
    "    \n",
    "    def __call__(self, value):\n",
    "        if value < (self.first_max - self.base_triangle):\n",
    "            return 0.0\n",
    "        elif value < self.first_max:\n",
    "            return 1.0 - ((self.first_max - value) * self.slope)\n",
    "        elif value < (self.first_max + self.width_top):\n",
    "            return 1.0\n",
    "        elif value < (self.first_max + self.width_top + self.base_triangle):\n",
    "            return 1.0 - ((value - (self.first_max + self.width_top)) * self.slope)\n",
    "        else:\n",
    "            return 0.0"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 8,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "image/png": "iVBORw0KGgoAAAANSUhEUgAAAXcAAAD7CAYAAACRxdTpAAAABHNCSVQICAgIfAhkiAAAAAlwSFlz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\nkkMa7iIiOaThLiKSQxruIiI59P8B0itwD9NcTCUAAAAASUVORK5CYII=\n",
      "text/plain": [
       "<matplotlib.figure.Figure at 0x7f80ea882a50>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "t = TrapezoidalMF(3, 6.5, 3.5)\n",
    "t.plot(0, 10)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "- Note that \"crisp\" and \"triangular\" membership functions can be represented by means of trapezoidal MFs."
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Exercise"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "- Test the functions above. Change the parameters of each membership funciton and observe the results.\n",
    "\n",
    "- Using the provided functions as starting point, implement a new trapezoidal membership function without the constraint of being isosceles. In order to test your implmentation, use your function to build a membership function with a trapezoidal shape: triangular at the left and rectangular at the right."
   ]
  },
  {
   "cell_type": "raw",
   "metadata": {},
   "source": [
    "                                            _____________\n",
    "                                           /             |\n",
    "                                          /              |\n",
    "                                         /               |\n",
    "                                  ______/                |_______"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## To further explore about membership functions... "
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Singleton MFs\n",
    "                                                 |\n",
    "                                                 |\n",
    "                                                 |\n",
    "                                  _______________|_______________\n",
    "                                  \n",
    "- A special membership function is the so-called \"singleton\", in which all the membership is \"concentrated\" in a single point.\n",
    "    - In your opinion, what is the meaning of such a function?\n",
    "    - Do you see an utility for the use of singleton membership functions?\n",
    "    \n",
    "### Smooth MFs\n",
    "- Smoother membership functions (e.g., Gaussian, exponential, or sigmoid) are also very common in fuzzy logic.\n",
    "    - In your opinion what could be the pros and cons of these smooth functions as compared to their edgier counterparts?\n",
    "\n",
    "### Arbitrary MFs\n",
    "- There is no formal need for a membership function to have a convex, coherent shape (as those explored until now). So, they could be as arbitrary as you want or need.\n",
    "    - In your opinion what could be the pros and cons of using arbitrary membership functions as compared to their well-behaved counterparts?\n"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 2",
   "language": "python",
   "name": "python2"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 2
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython2",
   "version": "2.7.9"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 0
}
